﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Controls;
using System.Windows.Shapes;
using System.Windows.Media;

namespace Festplattenübersicht
{
    abstract class FileSystemItem
    {
        protected string name;
        public string Name
        {
            get { return name; }
        }
        protected long size;
        public long Size
        {
            get { return size; }
        }

        public abstract void Draw(Canvas canvas, double left, double top, double width, double height, bool isVertical);
    }

    class Folder : FileSystemItem
    {
        List<FileSystemItem> content = new List<FileSystemItem>();

        public Folder(string path)
        {
            System.IO.DirectoryInfo di = new System.IO.DirectoryInfo(path);
            name = di.Name;
            System.IO.FileInfo[] files = di.GetFiles();
            foreach (var item in files)
            {
                content.Add(new File(item));
            }
            System.IO.DirectoryInfo[] directories = di.GetDirectories();
            foreach (var item in directories)
            {
                content.Add(new Folder(item.FullName));
            }

            // neu gegenüber Video
            content.Sort((x,y) => Math.Sign(y.Size - x.Size));

            //for (int i = 0; i < content.Count; i++)
            //{
            //    size += content[i].Size;
            //}
            //foreach (var item in content)
            //{
            //    size += item.Size;
            //}
            size = content.Sum(x => x.Size);
        }

        // neu gegenüber Video
        const double frameMargin = 1.0;
        const double fillMargin = 3.0; 

        public override void Draw(Canvas canvas,
            double left, double top, double width, double height,
            bool isVertical)
        {
            Rectangle r = new Rectangle(); // neu gegenüber Video
            r.Width = Math.Max(width - 2.0 * frameMargin, 0.0);
            r.Height = Math.Max(height - 2.0 * frameMargin, 0.0);
            r.Stroke = Brushes.Black;
            Canvas.SetLeft(r, left + frameMargin);
            Canvas.SetTop(r, top + frameMargin);
            canvas.Children.Add(r);
            r.ToolTip = Name + " " + Size + " Bytes";

            // eleganter als im Video
            left += fillMargin;
            top += fillMargin;
            width -= 2.0 * fillMargin;
            height -= 2.0 * fillMargin;
            if (width >= 0 && height >= 0) // Zu klein? Dann nichts zeichnen!
            {
                if (isVertical)
                {
                    double topSum = top;
                    foreach (var item in content)
                    {
                        double h = height * item.Size / Size;
                        item.Draw(canvas, left, topSum, width, h, false);
                        topSum += h;
                    }
                }
                else
                {
                    double leftSum = left;
                    foreach (var item in content)
                    {
                        double w = width * item.Size / Size;
                        item.Draw(canvas, leftSum, top, w, height, true);
                        leftSum += w;
                    }
                }
            }
        }
    }

    class File : FileSystemItem
    {
        public File(System.IO.FileInfo fileInfo)
        {
            name = fileInfo.Name;
            size = fileInfo.Length;
        }

        static Random random = new Random();

        public override void Draw(Canvas canvas, double left, double top, double width, double height, bool isVertical)
        {
            Rectangle r = new Rectangle();
            r.Width = width;
            r.Height = height;
            Color c = Color.FromRgb((byte)random.Next(50, 190), (byte)random.Next(50, 190), (byte)random.Next(50, 190));
            r.Fill = new SolidColorBrush(c);
            Canvas.SetLeft(r, left);
            Canvas.SetTop(r, top);
            canvas.Children.Add(r);
            r.ToolTip = Name + " " + Size + " Bytes"; // neu gegenüber Video
        }
    }
}
